import type { OpenId4VcCredentialHolderBinding, OpenId4VciCredentialOffer, OpenId4VciCredentialRequest, OpenId4VciCredentialSupported, OpenId4VciCredentialSupportedWithId, OpenId4VciIssuerMetadataDisplay } from '../shared';
import type { AgentContext, ClaimFormat, W3cCredential, SdJwtVcSignOptions } from '@credo-ts/core';
export interface OpenId4VciPreAuthorizedCodeFlowConfig {
    preAuthorizedCode?: string;
    userPinRequired?: boolean;
}
export type OpenId4VcIssuerMetadata = {
    issuerUrl: string;
    credentialEndpoint: string;
    tokenEndpoint: string;
    authorizationServer?: string;
    issuerDisplay?: OpenId4VciIssuerMetadataDisplay[];
    credentialsSupported: OpenId4VciCredentialSupported[];
};
export interface OpenId4VciCreateCredentialOfferOptions {
    offeredCredentials: string[];
    /**
     * baseUri for the credential offer uri. By default `openid-credential-offer://` will be used
     * if no value is provided. If a value is provided, make sure it contains the scheme as well as `://`.
     */
    baseUri?: string;
    preAuthorizedCodeFlowConfig: OpenId4VciPreAuthorizedCodeFlowConfig;
}
export interface OpenId4VciCreateCredentialResponseOptions {
    credentialRequest: OpenId4VciCredentialRequest;
    /**
     * You can optionally provide a credential request to credential mapper that will be
     * dynamically invoked to return credential data based on the credential request.
     *
     * If not provided, the `credentialRequestToCredentialMapper` from the agent config
     * will be used.
     */
    credentialRequestToCredentialMapper?: OpenId4VciCredentialRequestToCredentialMapper;
}
export type OpenId4VciCredentialRequestToCredentialMapper = (options: {
    agentContext: AgentContext;
    /**
     * The credential request received from the wallet
     */
    credentialRequest: OpenId4VciCredentialRequest;
    /**
     * The offer associated with the credential request
     */
    credentialOffer: OpenId4VciCredentialOffer;
    /**
     * Verified key binding material that should be included in the credential
     *
     * Can either be bound to did or a JWK (in case of for ex. SD-JWT)
     */
    holderBinding: OpenId4VcCredentialHolderBinding;
    /**
     * The credentials supported entries from the issuer metadata that were offered
     * and match the incoming request
     *
     * NOTE: in v12 this will probably become a single entry, as it will be matched on id
     */
    credentialsSupported: OpenId4VciCredentialSupported[];
}) => Promise<OpenId4VciSignCredential> | OpenId4VciSignCredential;
export type OpenId4VciSignCredential = OpenId4VciSignSdJwtCredential | OpenId4VciSignW3cCredential;
export interface OpenId4VciSignSdJwtCredential extends SdJwtVcSignOptions {
    format: ClaimFormat.SdJwtVc | `${ClaimFormat.SdJwtVc}`;
}
export interface OpenId4VciSignW3cCredential {
    format: ClaimFormat.JwtVc | `${ClaimFormat.JwtVc}` | ClaimFormat.LdpVc | `${ClaimFormat.LdpVc}`;
    verificationMethod: string;
    credential: W3cCredential;
}
export interface OpenId4VciCreateIssuerOptions {
    /**
     * Id of the issuer, not the id of the issuer record. Will be exposed publicly
     */
    issuerId?: string;
    credentialsSupported: OpenId4VciCredentialSupportedWithId[];
    display?: OpenId4VciIssuerMetadataDisplay[];
}
