"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureCredentialEndpoint = void 0;
const router_1 = require("../../shared/router");
const OpenId4VcIssuerService_1 = require("../OpenId4VcIssuerService");
const credentialRequest_1 = require("../util/credentialRequest");
const verifyAccessToken_1 = require("./verifyAccessToken");
function configureCredentialEndpoint(router, config) {
    router.post(config.endpointPath, async (request, response, next) => {
        const { agentContext, issuer } = (0, router_1.getRequestContext)(request);
        const openId4VcIssuerService = agentContext.dependencyManager.resolve(OpenId4VcIssuerService_1.OpenId4VcIssuerService);
        // Verify the access token (should at some point be moved to a middleware function or something)
        try {
            await (0, verifyAccessToken_1.verifyAccessToken)(agentContext, issuer, request.headers.authorization);
        }
        catch (error) {
            return (0, router_1.sendErrorResponse)(response, agentContext.config.logger, 401, 'unauthorized', error);
        }
        try {
            const credentialRequest = request.body;
            const issuanceSession = await openId4VcIssuerService.findIssuanceSessionForCredentialRequest(agentContext, {
                issuerId: issuer.issuerId,
                credentialRequest,
            });
            if (!issuanceSession) {
                const cNonce = (0, credentialRequest_1.getCNonceFromCredentialRequest)(credentialRequest);
                agentContext.config.logger.warn(`No issuance session found for incoming credential request with cNonce ${cNonce} and issuer ${issuer.issuerId}`);
                return (0, router_1.sendErrorResponse)(response, agentContext.config.logger, 404, 'invalid_request', null);
            }
            const { credentialResponse } = await openId4VcIssuerService.createCredentialResponse(agentContext, {
                issuanceSession,
                credentialRequest,
            });
            response.json(credentialResponse);
        }
        catch (error) {
            (0, router_1.sendErrorResponse)(response, agentContext.config.logger, 500, 'invalid_request', error);
        }
        // NOTE: if we don't call next, the agentContext session handler will NOT be called
        next();
    });
}
exports.configureCredentialEndpoint = configureCredentialEndpoint;
//# sourceMappingURL=credentialEndpoint.js.map